#ifdef QWS
#include <qpe/qpeapplication.h>
#else
#include <qapplication.h>
#endif

#include <cstring>
#include <qdom.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtimer.h>
#include "weatherwidget.h"

// Parametri proxy servera.
#define PROXY_HOST "proxy.rcub.bg.ac.yu"
#define PROXY_PORT 8080

// URL sa vremenskim izvjestajem za datu lokaciju.
#define WEATHER_REPORT_URL "http://xoap.weather.com/weather/local/YIXX0005?cc=*&prod=xoap&par=1007489912&key=1b4b59fa65258f8a&unit=m"

#ifdef QWS
// Direktorijum sa ikonama.
#define IMAGES_DIRECTORY "pics/weather"
#endif

WeatherWidget::WeatherWidget(QWidget * parent, const char *name)
        : QWidget(parent, name)
{
        // Postavlja se naslov prozora.
	setCaption(tr("Weather Report"));

        // Kreiraju se labele za ispis lokacije i vremena ocitavanja.
	locationLabel = new QLabel(tr("<h2>Unknown location</h2>"), this);
	timestampLabel = new QLabel(tr("Unknown time"), this);

        // Kreira se label za tekst o vremenskim prilikama.
	conditionTextLabel = new QLabel(tr("<h3>Unknown</h3>"), this);
	conditionTextLabel->
	    setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);

        // Kreira se label za ikonu vremenskih prilika.
	conditionIconLabel = new QLabel(this);
#ifdef QWS
	conditionIconLabel->
	    setPixmap(QPixmap
		      (QPEApplication::qpeDir() +
		       "/" IMAGES_DIRECTORY "/" + "na.png"));
#else
	conditionIconLabel->setPixmap(QPixmap::fromMimeSource("na.png"));
#endif
	conditionIconLabel->setAlignment(Qt::AlignHCenter);

        // Kreiraju se widget-i za ispis tekuce temperature.
	QLabel         *temperatureLabel =
	    new QLabel(tr("Temperature"), this);
	temperatureLineEdit = new QLineEdit(this);
	temperatureLineEdit->setReadOnly(true);
	temperatureLineEdit->setAlignment(Qt::AlignRight);

        // Kreiraju se widget-i za ispis subjektivne temperature.
	QLabel         *feelsLikeLabel =
	    new QLabel(tr("Feels Like"), this);
	feelsLikeLineEdit = new QLineEdit(this);
	feelsLikeLineEdit->setReadOnly(true);
	feelsLikeLineEdit->setAlignment(Qt::AlignRight);

        // Kreiraju se widget-i za ispis relativne vlaznosti vazduha.
	QLabel         *humidityLabel = new QLabel(tr("Humidity"), this);
	humidityLineEdit = new QLineEdit(this);
	humidityLineEdit->setReadOnly(true);
	humidityLineEdit->setAlignment(Qt::AlignRight);

        // Kreira se labela za ispis o izvoru informacija.
	QLabel         *suppliedByLabel =
	    new
	    QLabel(tr
		   ("<font size=-1>Weather info supplied by weather.com.</font>"),
		   this);
	suppliedByLabel->setAlignment(Qt::AlignHCenter);

        // Kreira se dugme za azuriranje informacija.
	QPushButton    *updateButton = new QPushButton(tr("Update"), this);
	updateButton->setFocus();
	connect(updateButton, SIGNAL(clicked()), this, SLOT(slotUpdate()));

        // Kreira se layout za centralne widget-e.
	QGridLayout    *middleLayout = new QGridLayout(5, 2);
	middleLayout->setSpacing(3);
	middleLayout->addWidget(conditionTextLabel, 0, 0);
	middleLayout->addWidget(conditionIconLabel, 0, 1);
	middleLayout->addWidget(temperatureLabel, 1, 0);
	middleLayout->addWidget(temperatureLineEdit, 1, 1);
	middleLayout->addWidget(feelsLikeLabel, 2, 0);
	middleLayout->addWidget(feelsLikeLineEdit, 2, 1);
	middleLayout->addWidget(humidityLabel, 3, 0);
	middleLayout->addWidget(humidityLineEdit, 3, 1);
	middleLayout->addMultiCellWidget(suppliedByLabel, 4, 4, 0, 1);

        // Kreira se layout za dugme za azuriranje informacija.
	QHBoxLayout    *bottomLayout = new QHBoxLayout;
	bottomLayout->setSpacing(3);
	bottomLayout->addStretch(1);
	bottomLayout->addWidget(updateButton);
	bottomLayout->addStretch(1);

        // Kreira se glavni layout.
	QVBoxLayout    *mainLayout = new QVBoxLayout(this);
	mainLayout->setMargin(5);
	mainLayout->setSpacing(3);
	mainLayout->addWidget(locationLabel);
	mainLayout->addWidget(timestampLabel);
	mainLayout->addLayout(middleLayout);
	mainLayout->addStretch(1);
	mainLayout->addLayout(bottomLayout);

        // Kreira se socket za preuzimanje izvjestaja o vremenu.
	socket = new QSocket(this);
	connect(socket, SIGNAL(connected()), this,
		SLOT(slotSocketConnected()));
	connect(socket, SIGNAL(readyRead()), this,
		SLOT(slotSocketReadyRead()));
	connect(socket, SIGNAL(connectionClosed()), this,
		SLOT(slotSocketConnectionClosed()));
	connect(socket, SIGNAL(error(int)), this,
		SLOT(slotSocketError(int)));
}

void
WeatherWidget::slotUpdate()
{
        // Brise se prethodni izvjestaj.
	report.truncate(0);

        // Pokrece se povezivanje sa serverom.
	socket->connectToHost(PROXY_HOST, PROXY_PORT);
}

void
WeatherWidget::slotSocketConnected()
{
        // Odasilje se zahtjev za izvjestajem serveru.
	char           *
	    request = "GET " WEATHER_REPORT_URL " HTTP/1.0\r\n\r\n";
	socket->writeBlock(request, strlen(request));
}

void
WeatherWidget::slotSocketReadyRead()
{
        // Nadodaje se novopristigli na postojeci dio izvjestaja.
	int
	    count = socket->bytesAvailable();
	char
	    buffer[count + 1];
	socket->readBlock(buffer, count);
	buffer[count] = 0;
	report += buffer;
}

void
WeatherWidget::slotSocketConnectionClosed()
{
        // U izvjestaju se trazi cc tag.
	QDomDocument
	    document;
	document.setContent(report.mid(report.find("<?xml")));
	QDomElement
	    root = document.documentElement();
	QDomNode
	    node;
	for (node = root.firstChild(); !node.isNull();
	     node = node.nextSibling())
		if (node.toElement().tagName() == "cc")
			break;
	if (node.isNull())
		return;

        // Prolazi se kroz elemente cc tag-a i azurira sadrzaj widget-a.
	for (node = node.firstChild(); !node.isNull();
	     node = node.nextSibling())
		if (node.toElement().tagName() == "obst")
			locationLabel->setText("<h2>" +
					       node.firstChild().toText().
					       data() + "</h2>");
		else if (node.toElement().tagName() == "lsup")
			timestampLabel->setText(node.firstChild().toText().
						data());
		else if (node.toElement().tagName() == "t")
			conditionTextLabel->setText("<h3>" +
						    node.firstChild().
						    toText().data() +
						    "</h3>");
		else if (node.toElement().tagName() == "icon")
#ifdef QWS
			conditionIconLabel->
			    setPixmap(QPixmap
				      (QPEApplication::qpeDir() +
				       "/" IMAGES_DIRECTORY "/" +
				       node.firstChild().toText().data() +
				       ".png"));
#else
			conditionIconLabel->
			    setPixmap(QPixmap::
				      fromMimeSource(node.firstChild().
						     toText().data() +
						     ".png"));
#endif
		else if (node.toElement().tagName() == "tmp")
			temperatureLineEdit->setText(node.firstChild().
						     toText().data() +
						     "C");
		else if (node.toElement().tagName() == "flik")
			feelsLikeLineEdit->setText(node.firstChild().
						   toText().data() + "C");
		else if (node.toElement().tagName() == "hmid")
			humidityLineEdit->setText(node.firstChild().
						  toText().data() + "%");
}

void
WeatherWidget::slotSocketError(int error)
{
        // Prikazuje se poruka o gresci.
	QString
	    message;
	switch (error) {
	case QSocket::ErrConnectionRefused:
		message = tr("connection refused");
		break;

	case QSocket::ErrHostNotFound:
		message = tr("host not found");
		break;

	case QSocket::ErrSocketRead:
		message = tr("socket read error");
		break;

	default:
		message = tr("unknown error");
	}
	QMessageBox::warning(this, tr("Error"),
			     tr
			     ("<p>Error while updating weather information: ")
			     + message);
}
